<?php
/**
 * WC_Gateway_Morune class
 *
 * @author   Morune <info@somewherewarm.gr>
 * @package  WooCommerce Morune Payments Gateway
 * @since    1.0.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Morune Gateway.
 *
 * @class    WC_Gateway_Morune
 * @version  1.0.7
 */
class WC_Gateway_Morune extends WC_Payment_Gateway {

	/**
	 * Payment gateway instructions.
	 * @var string
	 *
	 */
	protected $instructions;

	/**
	 * Whether the gateway is visible for non-admin users.
	 * @var boolean
	 *
	 */
	protected $hide_for_non_admin_users;

	/**
	 * Unique id for the gateway.
	 * @var string
	 *
	 */
	public $id = 'morune';

	/**
	 * Constructor for the gateway.
	 */
	public function __construct() {
		
		$this->icon               = apply_filters( 'woocommerce_morune_gateway_icon', '' );
		$this->has_fields         = false;
		$this->supports           = array(
			'pre-orders',
			'products',
		);

		$this->method_title       = _x( 'Платежный метод Morune', 'Платежный метод Morune', 'woocommerce-gateway-morune' );
		$this->method_description = __( 'Добавляет в WooCommerce возможность приёма платежей через платежную систему Morune', 'woocommerce-gateway-morune' );

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();


		// Define user set variables.
		$this->title                    = $this->get_option( 'title' );
		$this->description              = $this->get_option('description');
		$this->instructions             = $this->get_option( 'instructions', $this->description );
		$this->hide_for_non_admin_users = $this->get_option( 'hide_for_non_admin_users' );

		// Actions.

        if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
        } else {
            add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
        }

        add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'handle_callback'));
	}

	/**
	 * Initialise Gateway Settings Form Fields.
	 */
	public function init_form_fields() {

		$this->form_fields = array(
			'enabled' => array(
				'title'   => __( 'Enable/Disable', 'woocommerce' ),
				'type'    => 'checkbox',
				'label'   => __( 'Включить платежи Morune', 'woocommerce-gateway-morune' ),
				'default' => 'yes',
			),
			'hide_for_non_admin_users' => array(
				'type'    => 'checkbox',
				'label'   => __( 'Скрыть при оформлении заказа для пользователей без прав администратор', 'woocommerce-gateway-morune' ),
				'default' => 'no',
			),
			'title' => array(
				'title'       => __( 'Title', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Это контролирует заголовок, который пользователь видит во время оформления заказа.', 'woocommerce-gateway-morune' ),
				'default'     => _x( 'Оплата Morune', 'Платежный метод Morune', 'woocommerce-gateway-morune' ),
				'desc_tip'    => true,
			),
			'description' => array(
				'title'       => __( 'Description', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Описание способа оплаты, которое покупатель увидит при оформлении заказа.', 'woocommerce-gateway-morune' ),
				'default'     => __( 'Morune', 'woocommerce-gateway-morune' ),
				'desc_tip'    => true,
			),
            'merchant_id' => array(
                'title' => __('ID магазина', 'woocommerce-gateway-morune'),
                'type' => 'text',
            ),
            'secret_key' => array(
                'title' => __('Секретный ключ', 'woocommerce-gateway-morune') ,
                'type' => 'text',
            ),
            'webhook_key' => array(
                'title' => __('Дополнительный ключ', 'woocommerce-gateway-morune'),
                'type' => 'text',
            ),
            'switch_list' => array(
                'title' => __('Загружать список способов оплаты при оформлении заказа?', 'woocommerce-gateway-morune'),
                'type' => 'select',
                'options' => array(
                    '1' => __('Да', 'woocommerce-gateway-morune'),
                    '0' => __('Нет', 'woocommerce-gateway-morune'),
                ),
                'default' => '1',
            ),
		);
	}

	public function process_payment( $order_id ) {

		$order = wc_get_order( $order_id );

        try {
            $amount = $order->get_total();
            $amount = str_replace(',', '.', $amount);
            $amount = number_format($amount, 2, '.', '');

            $merchant_id = $this->get_option("merchant_id");

            $description = get_bloginfo('name') . ' – Заказ #' . $order_id;

            $currency = get_woocommerce_currency();

            $data = array(
                'order_id' => time() . '_' . $order_id,
                'amount' => $amount,
                'currency' => $currency,
                'shop_id' => $merchant_id,
                'comment' => $description,
            );
            if ($this->get_option("switch_list") && !empty($_COOKIE['wc_morune_service'])) {
                $data['include_service'] = array($_COOKIE['wc_morune_service']);
            }

            $url = $this->getPaymentUrl($data);

            WC()->cart->empty_cart();

            return array('result' => 'success', 'redirect' => $url);
        } catch (Throwable $exception) {
            $message = __( 'Ошибка создания инвойса.', 'woocommerce-gateway-morune' );
			$order->update_status( 'failed', $message );
            throw new Exception( $message );
        }
	}

    function getPaymentUrl($data)
    {
        require_once 'class-morune-merchant_api.php';

        $morune = new WC_Morune_Merchant_Api($this->get_option("secret_key"));

        $request = $morune->buildQuery('invoice/create', $data, 'post');

        if (!$request) {
            throw new Exception('Ошибка создания инвойса.');
        }

        $request = json_decode($request, true);

        return $request['data']['url'];
    }

    function handle_callback()
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            file_put_contents(__DIR__ . '/woomorune_error.log', date('Y-m-d H:i:s') . ' - callback # morune: Invalid method.' . "\r\n", FILE_APPEND);
            http_response_code(405); // Method Not Allowed
            exit;
        }

        $signature_header = $_SERVER['HTTP_X_API_SHA256_SIGNATURE'] ?? '';

        if (!$signature_header) {
            file_put_contents(__DIR__ . '../logs/morune_api_error.log', date('Y-m-d H:i:s') . ' - callback # morune: not signature.' . "\r\n", FILE_APPEND);
            exit;
        }

        $sign = $request_body = file_get_contents('php://input');
        file_put_contents(__DIR__ . '../logs/morune_api_error.log', date('Y-m-d H:i:s') . " - callback # morune: $request_body" . "\r\n", FILE_APPEND);

        $sign = json_decode($sign, true);
        ksort($sign);
        $sign = json_encode($sign);
        $secret_key = $this->get_option("webhook_key");
        $calculated_signature = hash_hmac('sha256', $sign, $secret_key);

        // Compare the calculated signature with the signature in the header
        if (!hash_equals($signature_header, $calculated_signature)) {
            file_put_contents(__DIR__ . '../logs/morune_api_error.log', date('Y-m-d H:i:s') . ' - callback # morune: Invalid signature.' . "\r\n", FILE_APPEND);
            http_response_code(401); // Unauthorized
            exit;
        }
        $request_body = json_decode($request_body, true);

        $order_id = $request_body['order_id'] ?? 0;

        if (!$order_id) {
            file_put_contents(__DIR__ . '../logs/morune_api_error.log', date('Y-m-d H:i:s') . ' - callback # morune: empty order_id.' . "\r\n", FILE_APPEND);
            http_response_code(406);
            exit;
        }

        $order_id = explode('_', $request_body['order_id']);
        $order_id = (int)($order_id[1] ?? 0);

        if (!$order_id) {
            file_put_contents(__DIR__ . '../logs/morune_api_error.log', date('Y-m-d H:i:s') . ' - callback # morune: incorrect order_id.' . "\r\n", FILE_APPEND);
            http_response_code(406);
            exit;
        }

        $status = $request_body['status'] ?? 'unknown';

        if ($status !== 'success') {
            file_put_contents(__DIR__ . '../logs/morune_api_error.log', date('Y-m-d H:i:s') . " - callback # morune: unsuccessful status for order $order_id - $status.\r\n", FILE_APPEND);
            exit;
        }

        $order = new WC_Order($order_id);
        $order->payment_complete();
    }

    function getPaymentMethods(): array
    {
        if (!$this->get_option("switch_list")) {
            return [];
        }

        require_once 'class-morune-merchant_api.php';
        $morune = new WC_Morune_Merchant_Api($this->get_option("secret_key"));

        $request = $morune->buildQuery('shops/' . $this->get_option('merchant_id') . '/payment-tariffs', array());

        if (!$request) {
            return [];
        }

        $data = json_decode($request, true);

        $amount = WC()->cart->get_cart_contents_total();
        $currency = get_woocommerce_currency();

        $method_list = [];

        foreach ($data['data']['tariffs'] as $item) {
            if ($item['status'] === "enabled" && $item['min_sum'] <= $amount && ($item['service'] !== 'card' || $item['currency'] === $currency)) {
                $commission = $item['percent'] * $item['user_percent'] / 100;

                $method_list[] = [
                    'service' => $item['service'],
                    'service_label' => $item['service_label'],
                    'currency' => $item['currency'],
                    'commission' => $commission,
                    'is_checked' => (!empty($_SESSION['service']) && $item['service'] === $_SESSION['service'])
                ];
            }
        }

        return $method_list;
    }
}
